@@ -2,12 +2,13 @@ import numpy as np
 import pandas as pd
 from joblib import Parallel, delayed
 import category_encoders as ce
+from sklearn.base import BaseEstimator
 from inspect import getmembers, isfunction
 from tsforest import metrics
 from tsforest.trend import TrendModel
 from tsforest.features import (compute_train_features, 
                                compute_predict_features,
-                               compute_lagged_feature)
+                               compute_lagged_predict_feature)
 from tsforest.config import (calendar_features_names,
                              calendar_cyclical_features_names)
 from sklearn.base import TransformerMixin
@@ -24,16 +25,13 @@ AVAILABLE_TIME_FEATURES = ["year", "quarter", "month", "days_in_month",
                            "week_day_cos", "week_day_sin", "year_day_cos", "year_day_sin",
                            "year_week_cos", "year_week_sin", "month_cos", "month_sin"]
 
-# available methods in pandas.core.window.Rolling as of pandas 0.25.1 
+# available methods in pandas.core.window.Rolling as of pandas 1.0.4 
 AVAILABLE_RW_FUNCTIONS = ["count", "sum", "mean", "median", "var", "std", "min", 
                           "max", "corr", "cov", "skew", "kurt", "quantile"]
 
 AVAILABLE_METRICS = [member[0].split('_')[1] for member in getmembers(metrics) 
                      if isfunction(member[1])]
 
-AVAILABLE_ENCODERS = [member[1] for member in getmembers(ce)
-                      if member[0]=="__all__"][0] + ["default"]
-
 class ForecasterBase(object):
     """
     Parameters
@@ -120,8 +118,22 @@ class ForecasterBase(object):
         if not isinstance(self.categorical_features, dict):
             raise TypeError("Parameter 'categorical_features' should be of type 'dict'.")
         else:
-            if any([encoding not in AVAILABLE_ENCODERS for encoding in self.categorical_features.values()]):
-                raise ValueError(f"Values in 'categorical_features' should be any of: {AVAILABLE_ENCODERS}")
+            for encoding in self.categorical_features.values():
+                if isinstance(encoding, str):
+                    if encoding != "default":
+                        raise ValueError(f"Unrecognized value: {encoding} for categorical feature.")
+                elif isinstance(encoding, tuple):
+                    if len(encoding) != 3:
+                        raise ValueError(f"Encoding tuple should by of size 3.")
+                    target,encoder_class,encoder_kwargs = encoding
+                    if not isinstance(target, str):
+                        raise TypeError("Target used for encoding should be of type 'str'.")
+                    if not issubclass(encoder_class, BaseEstimator):
+                        raise TypeError("Encoder class should inherit from 'sklearn.base import BaseEstimator'.")
+                    if (encoder_kwargs is not None) and (not isinstance(encoder_kwargs, dict)):
+                        raise TypeError("Encoder kwargs should by of type 'dict'.")
+                else:
+                    raise ValueError(f"Values in 'categorical_features' should be of type 'str' or 'tuple'.")
 
         if not isinstance(self.calendar_anomaly, list):
             raise TypeError("Parameter 'calendar_anomaly' should be of type 'list'.")
@@ -210,18 +222,23 @@ class ForecasterBase(object):
         for feature,encoding in self.categorical_features.items():
             if encoding == "default": 
                 if not np.issubdtype(train_features[feature].dtype, np.number):
-                    encoding = "OrdinalEncoder"
+                    target = "y_raw"
+                    encoder_class = ce.OrdinalEncoder
+                    encoder_kwargs = dict()
                 else: continue
-            encoder_class = getattr(ce, encoding)
-            encoder = encoder_class(cols=[feature])
-            encoder.fit(train_features.loc[:, [feature]], train_features.loc[:, "y_raw"].values)
+            else: 
+                target,encoder_class,encoder_kwargs = encoding
+                if target == "y": target = "y_raw"
+                if encoder_kwargs is None: encoder_kwargs = dict()
+            encoder = encoder_class(cols=[feature], **encoder_kwargs)
+            encoder.fit(train_features.loc[:, [feature]], train_features.loc[:, target].values)
             transformed = encoder.transform(train_features.loc[:, [feature]])
             if feature in self.ts_uid_columns:
                 train_features["_"+feature] = transformed.values
                 self.exclude_features.append(feature)
             else:
                 del train_features[feature]
-                train_features[feature] = transformed.values
+                train_features[transformed.columns] = transformed.values
             categorical_encoders[feature] = encoder
         return train_features,categorical_encoders
     
@@ -308,7 +325,8 @@ class ForecasterBase(object):
                                                 window_shifts=self.window_shifts,
                                                 window_sizes=self.window_sizes,
                                                 window_functions=self.window_functions,
-                                                ignore_const_cols=True)
+                                                ignore_const_cols=True,
+                                                n_jobs=self.n_jobs)
         if "zero_response" in train_features.columns:
             train_features = train_features.query("zero_response != 1")
         if "calendar_anomaly" in train_features.columns:
@@ -344,8 +362,6 @@ class ForecasterBase(object):
         else:
             train_data["y_raw"] = train_data["y"].values
         train_features = self.prepare_train_features(train_data)
-        # needed to keep track of the rows used in valid_index
-        train_features.set_index(train_data.index, inplace=True)
 
         if len(valid_index) > 0:
             valid_features = train_features.loc[valid_index, :]
@@ -509,7 +525,7 @@ class ForecasterBase(object):
         predict_features.set_index(["ds"] + self.ts_uid_columns, drop=False, inplace=True)
         
         with Parallel(n_jobs=self.n_jobs) as parallel:
-            delayed_func = delayed(compute_lagged_feature)
+            delayed_func = delayed(compute_lagged_predict_feature)
 
             for time_step in np.sort(predict_features.ds.unique()):
                 lag_kwargs = [{"lag":lag} for lag in self.lags]   
